/* socket.c */

#include "config.h"

#include <unistd.h>
#include <string.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>
#include <glib.h>

#include "logger.h"
#include "socket.h"


gboolean process_sock_req( GIOChannel *io, GIOCondition cond, SocketData *data )
{
   int sockfd, connfd;
   size_t nadr = sizeof(struct sockaddr_in);
   
   log_debug(__FILE__": process_sock_req called (cond %#06x)", cond);
   
   sockfd = g_io_channel_unix_get_fd(io);
   if( (connfd=accept(sockfd, (struct sockaddr *)&data->adr, &nadr)) >= 0 )
   {
      data->chcon = g_io_channel_unix_new(connfd);
      g_io_add_watch(data->chcon, G_IO_IN|G_IO_ERR|G_IO_HUP,
                     data->iofunc, data);
   }
   
   return TRUE;
}

SocketData* socket_new_server( gint port, GIOFunc iofunc )
{
   SocketData *data;
   int sockfd, i;
   
   data = g_new0(SocketData, 1);
   data->adr.sin_family = AF_INET;
   data->adr.sin_port = htons(port);
   memset(&data->adr.sin_addr, 0, sizeof(data->adr.sin_addr));
   data->nadr = sizeof(data->adr);
   data->chcon = NULL;
   data->iofunc = iofunc;
   
   if( (sockfd=socket(PF_INET, SOCK_STREAM, 0)) < 0 )
      log_error("Error in creating socket: %s", g_strerror(errno));
   
   i = 0;
   setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
   
   if( bind(sockfd, (struct sockaddr *)&data->adr, data->nadr) < 0 )
      log_error("Error in bind: %s", g_strerror(errno));
   
   if( listen(sockfd, 1) < 0 )
      log_error("Error in listen: %s", g_strerror(errno));
      
   data->chlis = g_io_channel_unix_new(sockfd);
   g_io_add_watch(data->chlis, G_IO_IN|G_IO_ERR|G_IO_HUP,
                  (GIOFunc)process_sock_req, data);
   
   return data;
}

SocketData* socket_new_client( gchar *hostname, gint port )
{
   SocketData *data;
   struct in_addr inadr;
   struct hostent *rechner;
   int sockfd;
   
   data = g_new0(SocketData, 1);
   data->adr.sin_family = AF_INET;
   data->adr.sin_port = htons(port);
   data->nadr = sizeof(data->adr);
   
   if( inet_aton(hostname, &inadr) )
      rechner = gethostbyaddr( (char*) &inadr, sizeof(inadr), AF_INET);
   else
      rechner = gethostbyname(hostname);
      
   if( rechner == NULL )
      log_error("Error in resolving hostname '%s'", hostname);
   
   memcpy(&data->adr.sin_addr, rechner->h_addr_list[0], sizeof(data->adr.sin_addr));
   
   if( (sockfd=socket(PF_INET, SOCK_STREAM, 0)) < 0 )
      log_error("Error in creating socket: %s", g_strerror(errno));
   
   if( connect(sockfd, (struct sockaddr *)&data->adr, data->nadr) < 0 )
      log_error("Error in connecting to server: %s", g_strerror(errno));
   
   data->chcon = g_io_channel_unix_new(sockfd);
   data->chlis = data->chcon;
   
   return data;
}

gint socket_write( SocketData *data, gchar *msg, gsize nmsg )
{
   guint n;
   
   g_return_val_if_fail( data != NULL, -1 );
   g_return_val_if_fail( data->chcon != NULL, -1 );
   
   if( g_io_channel_write(data->chcon, msg, nmsg, &n) != G_IO_ERROR_NONE )
      return -1;
   
   if( n != nmsg )
      return -1;
   
   return 0;
}

void socket_free( SocketData *data )
{
   g_io_channel_close(data->chlis);
   
   g_free(data);
}
